#define scr_create_vb_model
/*
This script merges 2 model frames into 1 vertex buffer which can perform a vertex interpolation
*/


var f1 = argument0;//frame 1
var f2 = argument1;//frame 2


var model = vertex_create_buffer();
vertex_begin(model, global.animation_format);

var segment = 8;//equivalent of ds_list_add() calls in "scr_load_model_blender_as_list"
//this needs to be the number of "ds_list_add" function calls per vertex in the "scr_load_model_blender_as_list" script


var size = ds_list_size(f1);

//merging 2 models into 1 vertex buffer
var i = 0;
repeat(size/segment){
    vertex_position_3d(model,ds_list_find_value(f1,i),-1*ds_list_find_value(f1,i+1),ds_list_find_value(f1,i+2));
    vertex_texcoord(model,ds_list_find_value(f1,i+3),-1*ds_list_find_value(f1,i+4));
    vertex_normal(model,ds_list_find_value(f1,i+5),-1*ds_list_find_value(f1,i+6),ds_list_find_value(f1,i+7));
   
    vertex_float3(model,ds_list_find_value(f2,i),-1*ds_list_find_value(f2,i+1),ds_list_find_value(f2,i+2));//position 2
    vertex_float3(model,ds_list_find_value(f2,i+5),-1*ds_list_find_value(f2,i+6),ds_list_find_value(f2,i+7));//normal2
       i+=+segment;
       

       
       
}


vertex_end(model);
vertex_freeze(model);

return model;

#define scr_create_animation_vertex_format
/* Creates the animation vertex format
*/

vertex_format_begin();

    vertex_format_add_position_3d();
    vertex_format_add_textcoord();
    vertex_format_add_normal();
    vertex_format_add_custom(vertex_type_float3,vertex_usage_colour)//x2,y2,z2
    vertex_format_add_custom(vertex_type_float3,vertex_usage_colour)//xn2,yn2,zn2
 

global.animation_format = vertex_format_end();

#define scr_load_animation
/*
This script loads an animation and returns a list with all animation frames
*/


var animation_path = argument[0];//animation path (example: "./animation/running"
var animation_looped = argument[1]


var animation_frame_count = 0;


//---------------------------------------------

//Loading every model frame

var animationFile = buffer_create(1,buffer_grow,1);
buffer_load_ext(animationFile,animation_path,0);//file_text_open_read(animation_path);
buffer_seek(animationFile, buffer_seek_start, 0);

//---- read Header -------

var key = buffer_read_chars(animationFile,9);
if(key != "BinAniObj"){
    show_message_async("Error!#"+string(animation_path)+"#is not a BAO file!");
}
var version = buffer_read(animationFile,buffer_u8);

if(version != 0){
    show_message_async("Error! Trying to load a different version of the bao-file!#"+string(animation_path)+"#> version-number: "+string(version));
}
var triangleCount = buffer_read(animationFile,buffer_u16);


//----------------------------

var model_frame;
var lastByte = 0;
do{
    model_frame[animation_frame_count] = scr_load_animation_frame_data(triangleCount,animationFile);//loads frame
    animation_frame_count+=+1;
    lastByte = buffer_read(animationFile, buffer_u8);
    //additional error check to give feedback if the .bao file is corrupted. (can happen if the original .obj frames weren't exported correctly.)
    if(lastByte != 254 && lastByte != 255){
        show_message_async("Byte alignment error occured. The .bao file is invalid.");
    }
}until(lastByte == 255);//the last byte of a .bao file is always 255.
buffer_delete(animationFile);

//---------------------------------------------

//converting the model frames to the custom vertex format with animation support
var animation = ds_list_create();//final animation is stored in this list
for(var i = 0;i<animation_frame_count-1;i++){
   ds_list_add(animation,scr_create_vb_model(model_frame[i],model_frame[i+1]));


}


if(animation_looped == true or animation_frame_count == 1){//if( for some reason) the animation contains only 1 frame, then add this first frame to this animation a second time in order to bypass an error.
    ds_list_add(animation,scr_create_vb_model(model_frame[animation_frame_count-1],model_frame[0]));//last animation interpolates between last model frame and first model frame
    
}


//---------------------------------------------

//delete now the original model frames which aren't needed anymore
for(var j = 0;j<animation_frame_count;j++){
    ds_list_destroy(model_frame[j]);
}

//---------------------------------------------

return animation;

#define scr_load_animation_frame_data
//include normals. include UVs, triangulate triangles!



var triangles,v_listX,v_listY,v_listZ,vt_listX,vt_listY,vn_listX,vn_listY,vn_listZ;

var tri_count = argument[0];

var buffer;
buffer = argument[1];




v_listX = ds_list_create();
v_listY = ds_list_create();
v_listZ = ds_list_create();

vt_listX = ds_list_create();
vt_listY = ds_list_create();


vn_listX = ds_list_create();
vn_listY = ds_list_create();
vn_listZ = ds_list_create();


var model = ds_list_create();



var vert_count = buffer_read(buffer, buffer_u32);
var textureCoord_count = buffer_read(buffer, buffer_u32);
var normalCount = buffer_read(buffer, buffer_u32);


var lines = tri_count+vert_count+textureCoord_count+normalCount;//calculate how many informations were saved for the current frame


var current_triangle = 0;

for(var i=0;i<lines;i++){
    var info_id = buffer_read(buffer,buffer_u8);
    switch(info_id){
        case 0://vertices

            ds_list_add(v_listX,buffer_read(buffer,buffer_f32));
            ds_list_add(v_listY,buffer_read(buffer,buffer_f32));
            ds_list_add(v_listZ,buffer_read(buffer,buffer_f32));
        break;
        
        case 1://uv
            ds_list_add(vt_listX,buffer_read(buffer,buffer_f32));
            ds_list_add(vt_listY,buffer_read(buffer,buffer_f32));
        break;
        
        case 2://normals
            ds_list_add(vn_listX,buffer_read(buffer,buffer_f32));
            ds_list_add(vn_listY,buffer_read(buffer,buffer_f32));
            ds_list_add(vn_listZ,buffer_read(buffer,buffer_f32));
        break;
        

        case 3:
            repeat(3){
                
                var e1 = buffer_read(buffer, buffer_u32);//vertex nr
                var e2 = buffer_read(buffer, buffer_u32);//texture nr
                var e3 = buffer_read(buffer, buffer_u32);//normal nr
                    e1 = real(e1)-1;
                    e3 = real(e3)-1;
                    e2 = real(e2)-1;

                ds_list_add(model,ds_list_find_value(v_listX,e1));//x
                ds_list_add(model,ds_list_find_value(v_listY,e1));//y
                ds_list_add(model,ds_list_find_value(v_listZ,e1));//z
                
                ds_list_add(model,ds_list_find_value(vt_listX,e2));//u
                ds_list_add(model,ds_list_find_value(vt_listY,e2));//v
                
                ds_list_add(model,ds_list_find_value(vn_listX,e3));//xn
                ds_list_add(model,ds_list_find_value(vn_listY,e3));//xn
                ds_list_add(model,ds_list_find_value(vn_listZ,e3));//xn
            }
        break;

        

    }
    
}


//deletes lists
ds_list_destroy(v_listX);
ds_list_destroy(v_listY);
ds_list_destroy(v_listZ);
ds_list_destroy(vt_listX);
ds_list_destroy(vt_listY);
ds_list_destroy(vn_listX);
ds_list_destroy(vn_listY);
ds_list_destroy(vn_listZ);




return model;

#define scr_draw_animation_frame
/*
    This script draws the animation and calculates the interpolation variable
*/

var animation_list = argument0;//animation list which contains preprocessed vertex-buffers with all animation frames
var animation_step = argument1; //current animation step (must be between 0 and "animation_length");
// --> [0 <= animation_step < animation_length]   >>> animation_step must be below animation_length!!!
var animation_length = argument2;//maximum animation step 
var tex = argument3;      

     
    var pc = clamp((animation_step/animation_length),0,1);//calculate (in percent from 0 to 1) the animation progress (from 0 to 1)
    var frame_count = ds_list_size(animation_list); 
    var curr_frame = pc*frame_count;
    
    var intpl = curr_frame-floor(curr_frame);
    if(curr_frame >=frame_count){
        curr_frame = frame_count-1;
        intpl = 1;
        
    }
    
    shader_set_uniform_f(global.shader_interpol, intpl);//calculate the interpolation value from the current frame to the next frame
    
    var current_frame = floor(curr_frame); //calculate the current frame
    vertex_submit(ds_list_find_value(animation_list,current_frame),pr_trianglelist,tex);//submit the model

#define scr_delete_animation
//This script destroys a 3D animation.

var model = argument0;//the pointer to your loaded 3D animation.

for(i=0;i<ds_list_size(model);i++){
    vertex_delete_buffer(ds_list_find_value(model,i));//deletes each frame
}

ds_list_destroy(model);//destroys the list

#define buffer_read_chars
var anz,buff;
buff = argument0;
anz = argument1;
var ret;
ret = "";
var h;
for(h=1;h<=anz;h+=+1){ 
    ret=ret+chr(buffer_read(buff, buffer_u8));
}
return ret;

#define scr_TranspoInverse
var M = argument0;

var NormalMatrix;
    NormalMatrix[0] = (M[5] * M[10] - M[6] * M[9]);
    NormalMatrix[4] = (M[2] * M[9] - M[1] * M[10]);
    NormalMatrix[8] = (M[1] * M[6] - M[2] * M[5]);
    NormalMatrix[1] = (M[6] * M[8] - M[4] * M[10]);
    NormalMatrix[5] = (M[0] * M[10] - M[2] * M[8]);
    NormalMatrix[9] = (M[2] * M[4] - M[0] * M[6]);
    NormalMatrix[2] = (M[4] * M[9] - M[5] * M[8]);
    NormalMatrix[6] = (M[1] * M[8] - M[0] * M[9]);
    NormalMatrix[10] = (M[0] * M[5] - M[1] * M[4]);

    
    NormalMatrix[3] = M[0];
    NormalMatrix[7] = M[0];
    NormalMatrix[5] = M[0];
    NormalMatrix[11] = M[0];
    NormalMatrix[12] = M[0];
    NormalMatrix[13] = M[0];
    NormalMatrix[14] = M[0];
    NormalMatrix[15] = M[0];
    
return NormalMatrix;


